@@ -402,7 +402,7 @@ localstatus TStdLogicDS::performStartSync(void)
     sta=implEndDataRead();
     // show items
     PDEBUGPRINTFX(DBG_HOT,("%s: number of local items involved in %ssync = %ld",getName(), fSlowSync ? "slow " : "",(long)fItems.size()));
-    CONSOLEPRINTF(("  %ld local items are new/changed/deleted for this sync",fItems.size()));
+    CONSOLEPRINTF(("  %ld local items are new/changed/deleted for this sync",(long)fItems.size()));
     if (PDEBUGTEST(DBG_DATA+DBG_DETAILS)) {
       PDEBUGBLOCKFMTCOLL(("SyncSet","Items involved in Sync","datastore=%s",getName()));
       for (TSyncItemPContainer::iterator pos=fItems.begin();pos!=fItems.end();pos++) {
@@ -587,8 +587,8 @@ localstatus TStdLogicDS::startDataAccessForServer(void)
         // this will not work, warn (but no longer abort session, as Siemens S55 guys don't like that)
         CONSOLEPRINTF((
           "Warning: Synchronisation involves more items (%ld) than client can possibly manage (%ld",
-          (sInt32)fItems.size(),
-          (sInt32)getRemoteDatastore()->getMaxID()
+          (long)fItems.size(),
+          (long)getRemoteDatastore()->getMaxID()
         ));
         PDEBUGPRINTFX(DBG_ERROR,(
           "Warning: Synchronisation involves more items (%ld) than client can possibly manage (%ld)",
@@ -611,7 +611,7 @@ localstatus TStdLogicDS::startDataAccessForServer(void)
 // called to check if conflicting replace or delete command from server exists
 TSyncItem *TStdLogicDS::getConflictingItemByRemoteID(TSyncItem *syncitemP)
 {
-  // search for conflicting item by LUID
+  // search for conflicting item by remoteID
   TSyncItemPContainer::iterator pos;
   for (pos=fItems.begin(); pos!=fItems.end(); ++pos) {
     if (strcmp((*pos)->getRemoteID(),syncitemP->getRemoteID())==0) {
@@ -630,6 +630,31 @@ TSyncItem *TStdLogicDS::getConflictingItemByRemoteID(TSyncItem *syncitemP)
 } // TStdLogicDS::getConflictingItemByRemoteID
 
 
+
+// called to check if conflicting item (with same localID) already exists in the list of items
+// to be sent to the server
+TSyncItem *TStdLogicDS::getConflictingItemByLocalID(TSyncItem *syncitemP)
+{
+  // search for conflicting item by localID
+  TSyncItemPContainer::iterator pos;
+  for (pos=fItems.begin(); pos!=fItems.end(); ++pos) {
+    if (strcmp((*pos)->getLocalID(),syncitemP->getLocalID())==0) {
+      // same LUID exists in data from server
+      PDEBUGPRINTFX(DBG_DATA+DBG_CONFLICT,(
+        "TStdLogicDS::getConflictingItemByLocalID, found RemoteID='%s', LocalID='%s', syncop=%s",
+        syncitemP->getRemoteID(),
+        syncitemP->getLocalID(),
+        SyncOpNames[syncitemP->getSyncOp()]
+      ));
+      return (*pos); // return pointer to item in question
+    }
+  }
+  PDEBUGPRINTFX(DBG_DATA+DBG_CONFLICT,("TStdLogicDS::getConflictingItemByLocalID, no conflicting item"));
+  return NULL;
+} // TStdLogicDS::getConflictingItemByLocalID
+
+
+
 // called to check if content-matching item from server exists for slow sync
 TSyncItem *TStdLogicDS::getMatchingItem(TSyncItem *syncitemP, TEqualityMode aEqMode)
 {
@@ -684,7 +709,7 @@ void TStdLogicDS::dontSendItemAsServer(TSyncItem *syncitemP)
   for (pos=fItems.begin(); pos!=fItems.end(); ++pos) {
     if (*pos == syncitemP) {
       // it is in our list
-      PDEBUGPRINTFX(DBG_DATA+DBG_HOT,("Item with localID='%s' will NOT be sent to client (usually due to slowsync match)",syncitemP->getLocalID()));
+      PDEBUGPRINTFX(DBG_DATA+DBG_HOT,("Item with localID='%s' will NOT be sent to client (slowsync match / duplicate prevention)",syncitemP->getLocalID()));
       delete *pos; // delete item itself
       fItems.erase(pos); // remove from list
       break;
@@ -1081,6 +1106,73 @@ void TStdLogicDS::logicMarkOnlyUngeneratedForResume(void)
 
 
 
+/// helper to merge database version of an item with the passed version of the same item;
+/// does age comparison by default, with "local side wins" as fallback
+TMultiFieldItem *TStdLogicDS::mergeWithDatabaseVersion(TSyncItem *aSyncItemP, bool &aChangedDBVersion, bool &aChangedNewVersion)
+{
+  aChangedDBVersion = false;
+  aChangedNewVersion = false;
+
+  TStatusCommand dummy(fSessionP);
+  TMultiFieldItem *dbVersionItemP = (TMultiFieldItem *)newItemForRemote(aSyncItemP->getTypeID());
+  if (!dbVersionItemP) return NULL;
+  // - set IDs
+  dbVersionItemP->setLocalID(aSyncItemP->getLocalID());
+  dbVersionItemP->setRemoteID(aSyncItemP->getRemoteID());
+  // - result is always a replace (item exists in DB)
+  dbVersionItemP->setSyncOp(sop_wants_replace);
+  // - try to get from DB
+  bool ok=logicRetrieveItemByID(*dbVersionItemP,dummy);
+  if (ok && dummy.getStatusCode()!=404) {
+    // item found in DB, merge with original item
+    // TODO (?): make this configurable
+    TConflictResolution crstrategy = cr_newer_wins;
+
+    if (crstrategy==cr_newer_wins) {
+      sInt16 cmpRes = aSyncItemP->compareWith(
+        *dbVersionItemP,
+        eqm_nocompare,this
+        #ifdef SYDEBUG
+        ,PDEBUGTEST(DBG_CONFLICT+DBG_DETAILS) // show age comparisons only if we want to see details
+        #endif
+      );
+      if (cmpRes==-1) crstrategy=cr_server_wins;
+      else crstrategy=cr_client_wins;
+      PDEBUGPRINTFX(DBG_DATA,(
+        "Newer item determined: %s",
+        crstrategy==cr_client_wins ?
+        "Incoming item is newer and wins" :
+        "DB item is newer and wins"
+      ));
+      if (crstrategy==cr_client_wins) {
+        aSyncItemP->mergeWith(*dbVersionItemP, aChangedNewVersion, aChangedDBVersion, this);
+      } else {
+        dbVersionItemP->mergeWith(*aSyncItemP, aChangedDBVersion, aChangedNewVersion, this);
+      }
+      PDEBUGPRINTFX(DBG_DATA,(
+        "Merged incoming item (%s,relevant,%smodified) with version from database (%s,%s,%smodified)",
+        crstrategy==cr_client_wins ? "winning" : "loosing",
+        aChangedNewVersion ? "" : "NOT ",
+        crstrategy==cr_server_wins ? "winning" : "loosing",
+        aChangedDBVersion ? "to-be-replaced" : "to-be-left-unchanged",
+        aChangedDBVersion ? "" : "NOT "
+      ));
+    }
+  }
+  else {
+    // no item found, we cannot force a conflict
+    PDEBUGPRINTFX(DBG_ERROR,("Could not retrieve database version of item, DB status code = %hd",dummy.getStatusCode()));
+    delete dbVersionItemP;
+    dbVersionItemP=NULL;
+    return NULL;
+  }
+  return dbVersionItemP;
+} // TStdLogicDS::mergeWithDatabaseVersion
+
+
+
+
+
 // called to process incoming item operation
 // Method takes ownership of syncitemP in all cases
 bool TStdLogicDS::logicProcessRemoteItem(
